@@ -22,9 +22,8 @@ import (
 	"encoding/binary"
 	"errors"
 	"fmt"
-	"github.com/IrineSistiana/mosdns/v2/dispatcher/pkg/pool"
+	"github.com/IrineSistiana/mosdns/v3/dispatcher/pkg/pool"
 	"github.com/go-redis/redis/v8"
-	"github.com/miekg/dns"
 	"time"
 )
 
@@ -32,6 +31,8 @@ type RedisCache struct {
 	client *redis.Client
 }
 
+// NewRedisCache returns a Redis cache specified by the url.
+// For the format of this url, see redis.ParseURL.
 func NewRedisCache(url string) (*RedisCache, error) {
 	opt, err := redis.ParseURL(url)
 	if err != nil {
@@ -42,7 +43,7 @@ func NewRedisCache(url string) (*RedisCache, error) {
 	return &RedisCache{client: c}, nil
 }
 
-func (r *RedisCache) Get(ctx context.Context, key string, allowExpired bool) (v *dns.Msg, storedTime, expirationTime time.Time, err error) {
+func (r *RedisCache) Get(ctx context.Context, key string) (v []byte, storedTime, expirationTime time.Time, err error) {
 	b, err := r.client.Get(ctx, key).Bytes()
 	if err != nil {
 		if err == redis.Nil { // no such key in redis, suppress redis.Nil err.
@@ -56,22 +57,15 @@ func (r *RedisCache) Get(ctx context.Context, key string, allowExpired bool) (v
 		return nil, time.Time{}, time.Time{}, fmt.Errorf("failed to unpack redis data: %w", err)
 	}
 
-	if !allowExpired && expirationTime.Before(time.Now()) { // suppress expired value
-		return nil, time.Time{}, time.Time{}, nil
-	}
-
 	return m, storedTime, expirationTime, nil
 }
 
-func (r *RedisCache) Store(ctx context.Context, key string, v *dns.Msg, storedTime, expirationTime time.Time) (err error) {
+func (r *RedisCache) Store(ctx context.Context, key string, v []byte, storedTime, expirationTime time.Time) error {
 	if time.Now().After(expirationTime) {
 		return nil
 	}
 
-	data, err := packRedisData(storedTime, expirationTime, v)
-	if err != nil {
-		return err
-	}
+	data := packRedisData(storedTime, expirationTime, v)
 	defer pool.ReleaseBuf(data)
 
 	return r.client.Set(ctx, key, data, expirationTime.Sub(time.Now())).Err()
@@ -82,33 +76,21 @@ func (r *RedisCache) Close() error {
 	return r.client.Close()
 }
 
-// packRedisData packs storedTime, expirationTime and msg m into one byte slice.
+// packRedisData packs storedTime, expirationTime and v into one byte slice.
 // The returned []byte should be released by pool.ReleaseBuf().
-func packRedisData(storedTime, expirationTime time.Time, m *dns.Msg) ([]byte, error) {
-	wireMsg, mb, err := pool.PackBuffer(m)
-	if err != nil {
-		return nil, err
-	}
-	defer pool.ReleaseBuf(mb)
-
-	v := pool.GetBuf(8 + 8 + len(wireMsg))
-	binary.BigEndian.PutUint64(v[:8], uint64(storedTime.Unix()))
-	binary.BigEndian.PutUint64(v[8:16], uint64(expirationTime.Unix()))
-	copy(v[16:], wireMsg)
-	return v, nil
+func packRedisData(storedTime, expirationTime time.Time, v []byte) []byte {
+	buf := pool.GetBuf(8 + 8 + len(v))
+	binary.BigEndian.PutUint64(buf[:8], uint64(storedTime.Unix()))
+	binary.BigEndian.PutUint64(buf[8:16], uint64(expirationTime.Unix()))
+	copy(buf[16:], v)
+	return buf
 }
 
-func unpackRedisValue(b []byte) (storedTime, expirationTime time.Time, m *dns.Msg, err error) {
+func unpackRedisValue(b []byte) (storedTime, expirationTime time.Time, v []byte, err error) {
 	if len(b) < 16 {
 		return time.Time{}, time.Time{}, nil, errors.New("b is too short")
 	}
 	storedTime = time.Unix(int64(binary.BigEndian.Uint64(b[:8])), 0)
 	expirationTime = time.Unix(int64(binary.BigEndian.Uint64(b[8:16])), 0)
-
-	m = new(dns.Msg)
-	err = m.Unpack(b[16:])
-	if err != nil {
-		return time.Time{}, time.Time{}, nil, err
-	}
-	return storedTime, expirationTime, m, nil
+	return storedTime, expirationTime, b[16:], nil
 }
