@@ -19,13 +19,12 @@ package msg_matcher
 
 import (
 	"context"
-	"fmt"
-	"github.com/IrineSistiana/mosdns/v2/dispatcher/handler"
-	"github.com/IrineSistiana/mosdns/v2/dispatcher/pkg/matcher/domain"
-	"github.com/IrineSistiana/mosdns/v2/dispatcher/pkg/matcher/elem"
-	"github.com/IrineSistiana/mosdns/v2/dispatcher/pkg/matcher/netlist"
-	"github.com/IrineSistiana/mosdns/v2/dispatcher/pkg/utils"
+	"github.com/IrineSistiana/mosdns/v3/dispatcher/handler"
+	"github.com/IrineSistiana/mosdns/v3/dispatcher/pkg/matcher/domain"
+	"github.com/IrineSistiana/mosdns/v3/dispatcher/pkg/matcher/elem"
+	"github.com/IrineSistiana/mosdns/v3/dispatcher/pkg/matcher/netlist"
 	"github.com/miekg/dns"
+	"net"
 )
 
 type ClientIPMatcher struct {
@@ -37,17 +36,15 @@ func NewClientIPMatcher(ipMatcher netlist.Matcher) *ClientIPMatcher {
 }
 
 func (m *ClientIPMatcher) Match(_ context.Context, qCtx *handler.Context) (matched bool, err error) {
-	if qCtx.From() != nil {
-		ip := utils.GetIPFromAddr(qCtx.From())
-		if ip != nil {
-			if m.ipMatcher.Match(ip) {
-				return true, nil
-			}
-		} else {
-			return false, fmt.Errorf("internal err: client addr [%s] is invalid", qCtx.From())
-		}
+	var clientIP net.IP
+	if meta := qCtx.ReqMeta(); meta != nil {
+		clientIP = meta.ClientIP
 	}
-	return false, nil
+	if clientIP == nil {
+		return false, nil
+	}
+
+	return m.ipMatcher.Match(clientIP), nil
 }
 
 type QNameMatcher struct {
