@@ -28,14 +28,14 @@ import (
 	"encoding/binary"
 	"encoding/pem"
 	"fmt"
-	"github.com/IrineSistiana/mosdns/v2/dispatcher/handler"
+	"github.com/IrineSistiana/mosdns/v3/dispatcher/handler"
+	"github.com/IrineSistiana/mosdns/v3/dispatcher/pkg/pool"
 	"github.com/miekg/dns"
-	"io/ioutil"
 	"math/big"
 	"net"
+	"os"
 	"regexp"
 	"strings"
-	"sync"
 	"time"
 	"unsafe"
 )
@@ -50,8 +50,8 @@ func GetIPFromAddr(addr net.Addr) (ip net.IP) {
 		return v.IP
 	case *net.IPNet:
 		return v.IP
-	case *NetAddr:
-		return v.IP()
+	case *net.IPAddr:
+		return v.IP
 	default:
 		return parseIPFromAddr(addr.String())
 	}
@@ -66,34 +66,6 @@ func SplitSchemeAndHost(addr string) (protocol, host string) {
 	}
 }
 
-// NetAddr implements net.Addr interface.
-type NetAddr struct {
-	addr    string
-	network string
-
-	parseIPOnce sync.Once
-	ip          net.IP // will be non-nil if addr is an ip addr.
-}
-
-func NewNetAddr(addr string, network string) *NetAddr {
-	return &NetAddr{addr: addr, network: network}
-}
-
-func (n *NetAddr) Network() string {
-	return n.network
-}
-
-func (n *NetAddr) String() string {
-	return n.addr
-}
-
-func (n *NetAddr) IP() net.IP {
-	n.parseIPOnce.Do(func() {
-		n.ip = parseIPFromAddr(n.addr)
-	})
-	return n.ip
-}
-
 func parseIPFromAddr(s string) net.IP {
 	ipStr, _, err := net.SplitHostPort(s)
 	if err != nil {
@@ -115,14 +87,21 @@ func GetMsgKey(m *dns.Msg, salt uint16) (string, error) {
 
 // GetMsgKeyWithBytesSalt unpacks m and appends salt to the string.
 func GetMsgKeyWithBytesSalt(m *dns.Msg, salt []byte) (string, error) {
-	wireMsg, err := m.Pack()
+	wireMsg, buf, err := pool.PackBuffer(m)
 	if err != nil {
 		return "", err
 	}
+	defer pool.ReleaseBuf(buf)
+
 	wireMsg[0] = 0
 	wireMsg[1] = 0
-	wireMsg = append(wireMsg, salt...)
-	return BytesToStringUnsafe(wireMsg), nil
+
+	sb := new(strings.Builder)
+	sb.Grow(len(wireMsg) + len(salt))
+	sb.Write(wireMsg)
+	sb.Write(salt)
+
+	return sb.String(), nil
 }
 
 // GetMsgKeyWithInt64Salt unpacks m and appends salt to the string.
@@ -141,7 +120,7 @@ func BytesToStringUnsafe(b []byte) string {
 func LoadCertPool(certs []string) (*x509.CertPool, error) {
 	rootCAs := x509.NewCertPool()
 	for _, cert := range certs {
-		b, err := ioutil.ReadFile(cert)
+		b, err := os.ReadFile(cert)
 		if err != nil {
 			return nil, err
 		}
@@ -254,12 +233,3 @@ func BoolLogic(ctx context.Context, qCtx *handler.Context, fs []handler.Matcher,
 
 	return matched, nil
 }
-
-// IsIPAddr returns true is s is a IP address. s can contain ":port".
-func IsIPAddr(s string) bool {
-	host, _, err := net.SplitHostPort(s)
-	if err != nil {
-		return net.ParseIP(s) != nil
-	}
-	return net.ParseIP(host) != nil
-}
